#include <stdio.h>
#include <stdlib.h>

typedef struct ball{
	int color;
	struct ball * next;
} Ball; 

typedef struct {
	int nextColor;
	int nBalls;
	Ball * first;
} HoppeUrn; 

int init(HoppeUrn * hoppe) {
	hoppe->nextColor = 1;
	hoppe->nBalls = 1;
	hoppe->first = (Ball *) malloc (sizeof(Ball));
	if (hoppe->first != NULL) {
		hoppe->first->color = 0;
		hoppe->first->next = NULL;
		return 1;
	}
	return 0;
}

void addLast(HoppeUrn * hoppe, Ball * last) {
	Ball * itr = hoppe->first;

	if (itr == NULL)
		hoppe->first = last;
	else {
		while (itr->next != NULL) {
			itr = itr->next;
		}
		itr->next = last;
	}
}

void freeList(HoppeUrn * hoppe) {
	Ball * itr = hoppe->first, * tmp;

	while (itr != NULL) {
		tmp = itr;
		itr = itr->next;
		free(tmp);
	}
}

// a
void move(HoppeUrn * hoppe) {
	int i = 0, rnd;
	Ball * drawnBall = hoppe->first;
	Ball * newBall = (Ball *) malloc (sizeof(Ball));

	// Check if the memory allocation succeeded
	if (newBall == NULL)
		return;

	// Draw a ball from the urn
	rnd = rand() % hoppe->nBalls;
	for (;i < rnd; ++i)
		drawnBall = drawnBall->next;

	// Set the color of the new ball according to the Hoppe rules
	newBall->color = drawnBall->color == 0? hoppe->nextColor++:drawnBall->color;
	newBall->next = NULL;

	// Add the new ball to the urn
	addLast(hoppe, newBall);
	++hoppe->nBalls;
}

// b
void play(HoppeUrn * hoppe, int nMoves) {
	int i;

	for (i = 0; i < nMoves; ++i)
		move(hoppe);
}

// c
int * histogram(HoppeUrn * hoppe) {
	int * hist = (int *) calloc (hoppe->nextColor, sizeof(int));
	Ball * itr = hoppe->first;

	// Check if the memory allocation succeeded
	if (hist == NULL)
		return NULL;

	// Iterate through all the balls in the urn and update the histogram
	for(;itr != NULL; itr = itr->next)
		++hist[itr->color];

	return hist;
}

// not part of the question
int main() {
	HoppeUrn urn;
	int * hist, i;

	init(&urn);

	play(&urn, 1000);

	hist = histogram(&urn);

	if (hist != NULL) {
		for(i = 0; i < urn.nextColor; ++i)
			printf("color %d, balls %d\n", i, hist[i]);
		// free memory allocated for hist
		free(hist);
	}

	// free memory allocated for the list of balls
	freeList(&urn);

	return 0;
}